/*
Copyright 2006 Renaun Erickson (http://renaun.com)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

*/
/*
	Connection Monitor connects to FMSConnectionMonitor component and relays 
	latency, up bandwidth, and down bandwidth for the current NetConnection
*/
package com.renaun.fms
{

import flash.net.NetConnection;
import flash.utils.setInterval;
import flash.utils.clearInterval;
import mx.collections.ArrayCollection;
import com.renaun.fms.vo.StatObject;


public class ConnectionMonitor
{
	
	//--------------------------------------------------------------------------
	//
	//  Constructor
	//
	//--------------------------------------------------------------------------

	/**
	 *  ConnectionMonitor constructor
	 */
	public function ConnectionMonitor( name:String, interval:Number = 1 )
	{
		this.name = name;
		this.interval = interval;
		
	}

	/**
	 * 	Connect to FMSConnectionMonitor on the FMS server.  You must have the FMSConnectionMonitor component present.
	 * 
	 *  @param netConnection Connect to netConnection
	 */
	public function connect( netConnection:NetConnection ):void 
	{
		nc = netConnection;
		Logger.debug( "ConnectionMonitor: connect: nc.connected: " + nc.connected + " - client: " + nc.client );	
		// Connect to server side component
		netConnection.call( "FMSConnectionMonitor." + this.name + ".connect", null, this.interval * 1000 );
	}

	//--------------------------------------------------------------------------
	//
	//  Variables
	//
	//--------------------------------------------------------------------------

	//----------------------------------
	//  nc
	//----------------------------------

	/**
	 *  The NetConnection parameter
	 */
	private var nc:NetConnection;

	/**
	 *  Name of this ConnectionMonitor
	 */
	private var name:String;

	/**
	 *  Monitor Interval
	 */
	private var interval:Number;

	/**
	 *  setInterval reference for checkConnectionStatus
	 */
	private var intConnectionStatus:uint;

	/**
	 *  Latency value in milliseconds
	 */
	[Bindable]
	public var latency:Number;

	/**
	 *  Up bandwidth value in bits/seconds
	 */
	[Bindable]
	public var bandwidthUp:Number;

	/**
	 *  Down bandwidth value in bits/seconds
	 */
	[Bindable]
	public var bandwidthDown:Number;

	/**
	 *  Attached Camera for monitoring frams per second
	 */
	public var camera:Object;

	/**
	 *	@private
	 */
    private var currentFPS:Number = 0;

	/**
	 *	@private
	 */
    private var historySize:Number = 100;
    
    /**
     *	The history Data
     */
    [Bindable]
    public var historyData:ArrayCollection;
 
    
	/**
	 *	Set Video object the container will control
	 */
	[Bindable]
    public function set size( value:Number ):void
    {
    	historySize = value;
    }   
    public function get size():Number
    {
    	return historySize;
    }      

	//--------------------------------------------------------------------------
	//
	//  Methods
	//
	//--------------------------------------------------------------------------

	/**
	 * 	Clear Stats History
	 */
	public function clearHistory():void
	{
		historyData = null;
	}
	
	/**
	 * 	Close the connections and clear Intervals
	 */
	public function close():void
	{		
		// clear checkConnectionStatus setInterval call
		clearInterval( this.intConnectionStatus );
		
		// Close connection to server component
		nc.call( "FMSConnectionMonitor." + this.name + ".close", null );
	}

	/**
	 *  Function called by server component to update latency
	 */
	public function update( ping:Number, up:Number, down:Number ):void
	{
		Logger.debug( "ConnectionMonitor: update: " + latency + " - " + bandwidthUp + " - " + bandwidthDown );
		if( isNaN( ping ) )
			return;
		// In milliseconds
		latency = ping;
		// In bytes
		bandwidthUp = up;
		// In bytes
		bandwidthDown = down;

		if( camera != null )
			currentFPS = camera.currentFPS;

		if( historySize > 0 ) {
			if( historyData == null )
				historyData = new ArrayCollection();
			
			var obj:StatObject = new StatObject();
			obj.latency = latency;
			obj.bandwidthUp = bandwidthUp;
			obj.bandwidthDown = bandwidthDown;
			obj.fps = currentFPS;
			
			historyData.addItem( obj );
			while( historyData.length > historySize  )
				historyData.removeItemAt( 0 );
		}
	}
	
	/**
	 * 	Formatting function for Bandwidth rates, uses bit/seconds as the dimension.
	 * 
	 * 	@param value The rate value in bytes
	 */
	public function formatRate( value:Number ):String
	{
		// 8 bits per Byte
		var fixp:Object = this.formatNumber(value * 8000);
		if ( fixp.exponent == -3 )
			fixp.value = 0, fixp.exponent = 0;
		else if ( fixp.exponent == 0 )
			fixp.unit = "bit/s";
		else if ( fixp.exponent == 3 )
			fixp.unit = "kbit/s";
		else if ( fixp.exponent == 6 )
			fixp.unit = "mbit/s";
			
		return fixp.value + " " + fixp.unit;		
	}

	/**
	 * 	Formatting function for Latency time, uses milliseconds as the dimension.
	 * 
	 * 	@param value The rate value in milliseconds
	 */
	public function formatTime( value:Number ):String
	{
		// value is in milliseconds so divide by 1000
		var fixp:Object = this.formatNumber(value/1000);
		if ( fixp.exponent == -3 )
			fixp.unit = "msec";
		else if ( fixp.exponent == 0 )
			fixp.unit = "sec";
			
		return fixp.value + " " + fixp.unit;
	}

	/**
	 * 	@private
	 */
	private function formatNumber( value:Number ):Object 
	{
		var result:Object;
		if ( value < 0.001 )
			result = {value:0,exponent:0};
		else if ( value < 1 )
			result = {value:value*1000,exponent:-3};
		else if ( value < 1000 )
			result = {value:value,exponent:0};
		else if ( value < 1000000 )
			result = {value:value/1000,exponent:3};
		else if ( value < 1000000000 )
			result = {value:value/1000000,exponent:6};
		
		if ( result.value < 10 )
			result.value = (Math.round(result.value*100))/100;
		else if ( result.value < 100 )
			result.value = (Math.round(result.value*10))/10;
		else
			result.value = Math.round(result.value);
	
		return result;
	}
	
}

}